
class Email():
    """
    emailType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "email",
            id: str = None,
            domain: str = None) -> None:
        """
        Initialize Email instance.

        Args:
            tag (str, optional): XML tag. Defaults to "email".
            id (str, optional): Identifier. Defaults to None.
            domain (str, optional): Domain. Defaults to None.
        """
        self.tag: str = tag
        self.id: str = id
        self.domain: str = domain


from .extensions import *
from .link import Link
from .track_segment import *

class Track():
    """
    trkType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "trk",
            name: str = None,
            cmt: str = None,
            desc: str = None,
            src: str = None,
            link: Link = None,
            number: int = None,
            type: str = None,
            extensions: Extensions = None,
            trkseg: list[TrackSegment] = []):
        """
        Initialize Track instance.

        Args:
        tag (str, optional): XML tag. Defaults to "trk".
            name (str, optional): Name. Defaults to None.
            cmt (str, optional): Comment. Defaults to None.
            desc (str, optional): Description. Defaults to None.
            src (str, optional): Source. Defaults to None.
            link (Link, optional): Link. Defaults to None.
            number (int, optional): Number. Defaults to None.
            type (str, optional): Type. Defaults to None.
            extensions (Extensions, optional): Extensions. Defaults to None.
            trkseg (list[TrackSegment], optional): List of track segments. Defaults to [].
        """
        self.tag: str = tag
        self.name: str = name
        self.cmt: str = cmt
        self.desc: str = desc
        self.src: str = src
        self.link: Link = link
        self.number: int = number
        self.type: str = type
        self.extensions: Extensions = extensions
        self.trkseg: list[TrackSegment] = trkseg

    def project(self, projection: str):
        """
        Project segments.

        Args:
            projection (str): Projection.
        """
        for segment in self.trkseg:
            segment.project(projection)




class Link():
    """
    linkType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "link",
            href: str = None,
            text: str = None,
            type: str = None) -> None:
        """
        Initialize Link instance.

        Args:
            tag (str, optional): XML tag. Defaults to "link".
            href (str, optional): Hyper reference. Defaults to None.
            text (str, optional): Text. Defaults to None.
            type (str, optional): Type. Defaults to None.
        """
        self.tag: str = tag
        self.href: str = href
        self.text: str = text
        self.type: str = type
        




class Copyright():
    """
    copyrightType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "copyright",
            author: str = None,
            year: int = None,
            licence: str = None) -> None:
        """
        Initialize Copyright instance.

        Args:
            tag (str, optional): XML tag. Defaults to "copyright".
            author (str, optional): Author. Defaults to None.
            year (int, optional): Year. Defaults to None.
            licence (str, optional): Licence. Defaults to None.
        """
        self.tag: str = tag
        self.author: str = author
        self.year: int = year
        self.licence: str = licence
        


import logging
import pandas as pd
from datetime import datetime, timezone

from .metadata import Metadata
from .way_point import WayPoint
from .route import Route
from .track import Track
from .extensions import Extensions
from ..utils import haversine_distance, ramer_douglas_peucker

class Gpx():
    """
    gpxType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "gpx",
            creator: str = None,
            xmlns: str = None,
            version: str = None,
            xmlns_xsi: str = None,
            xsi_schema_location: list[str] = None,
            xmlns_gpxtpx: str = None,
            xmlns_gpxx: str = None,
            xmlns_gpxtrk: str = None,
            xmlns_wptx1: str = None,
            metadata: Metadata = None,
            wpt: list[WayPoint] = [],
            rte: list[Route] = [],
            tracks: list[Track] = [],
            extensions: Extensions = None) -> None:
        """
        Initialize Gpx instance.

        Args:
            tag (str, optional): XML tag. Defaults to "gpx".
            creator (str, optional): Creator. Defaults to None.
            xmlns (str, optional): XML xmlns. Defaults to None.
            version (str, optional): Version. Defaults to None.
            xmlns_xsi (str, optional): XML xmlns_xsi. Defaults to None.
            xsi_schema_location (list[str], optional): XML schema location. Defaults to None.
            xmlns_gpxtpx (str, optional): _description_. Defaults to None.
            xmlns_gpxx (str, optional): _description_. Defaults to None.
            xmlns_gpxtrk (str, optional): _description_. Defaults to None.
            xmlns_wptx1 (str, optional): _description_. Defaults to None.
            metadata (Metadata, optional): Metadata. Defaults to None.
            wpt (list[WayPoint], optional): Way points. Defaults to [].
            rte (list[Route], optional): Routes. Defaults to [].
            tracks (list[Track], optional): List of tracks. Defaults to [].
            extensions (Extensions, optional): Extensions. Defaults to [].
        """
        self.tag: str = tag
        self.creator: str = creator
        self.xmlns: str = xmlns
        self.version: str = version

        self.xmlns_xsi: str = xmlns_xsi
        self.xsi_schema_location: str = xsi_schema_location
        self.xmlns_gpxtpx: str = xmlns_gpxtpx
        self.xmlns_gpxx: str = xmlns_gpxx
        self.xmlns_gpxtrk: str = xmlns_gpxtrk
        self.xmlns_wptx1: str = xmlns_wptx1

        self.metadata: Metadata = metadata
        self.wpt:list[WayPoint] = wpt
        self.rte: list[Route] = rte
        self.tracks: list[Track] = tracks
        self.extensions: Extensions = extensions

    def name(self) -> str:
        """
        Return activity name.

        Returns:
            str: Activity name.
        """
        return self.tracks[0].name

    def nb_points(self) -> int:
        """
        Compute the number of points in the GPX.

        Returns:
            int: Number of points in the GPX.
        """
        nb_pts = 0
        for track in self.tracks:
            for track_segment in track.trkseg:
                nb_pts += len(track_segment.trkpt)
        return nb_pts
    
    def bounds(self) -> tuple[float, float, float, float]:
        """
        Find minimum and maximum latitude and longitude.

        Returns:
            tuple[float, float, float, float]: Min latitude, min longitude, max latitude, max longitude
        """
        min_lat = self.tracks[0].trkseg[0].trkpt[0].lat
        min_lon = self.tracks[0].trkseg[0].trkpt[0].lon
        max_lat = min_lat
        max_lon = min_lon

        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    if track_point.lat < min_lat:
                        min_lat = track_point.lat
                    if track_point.lon < min_lon:
                        min_lon = track_point.lon
                    if track_point.lat > max_lat:
                        max_lat = track_point.lat
                    if track_point.lon > max_lon:
                        max_lon = track_point.lon
        return min_lat, min_lon, max_lat, max_lon


    def center(self) -> tuple[float, float]:
        """
        Compute the center coordinates of the track.

        Returns:
            tuple[float, float]: Latitude and longitude of the center point.
        """
        min_lat, min_lon, max_lat, max_lon = self.bounds()
        center_lat = min_lat + (max_lat - min_lat) / 2
        center_lon = min_lon + (max_lon - min_lon) / 2
        return center_lat, center_lon

    def distance(self) -> float:
        """
        Compute the total distance (meters) of the tracks contained in the Gpx element.

        Returns:
            float: Distance (meters)
        """
        dst = 0
        previous_point = self.tracks[0].trkseg[0].trkpt[0]
        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    dst += haversine_distance(previous_point, track_point)
                    previous_point = track_point
        return dst
    
    def ascent(self) -> float:
        """
        Compute the total ascent (meters) of the tracks contained in the Gpx element.

        Returns:
            float: Ascent (meters)
        """
        ascent = 0
        previous_elevation = self.tracks[0].trkseg[0].trkpt[0].ele
        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    if track_point.ele > previous_elevation:
                        ascent += track_point.ele - previous_elevation
                    previous_elevation = track_point.ele
        return ascent
    
    def descent(self) -> float:
        """
        Compute the total descent (meters) of the tracks contained in the Gpx element.

        Returns:
            float: Descent (meters)
        """
        descent = 0
        previous_elevation = self.tracks[0].trkseg[0].trkpt[0].ele
        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    if track_point.ele < previous_elevation:
                        descent += previous_elevation - track_point.ele
                    previous_elevation = track_point.ele
        return descent
    
    def min_elevation(self) -> float:
        """
        Compute minimum elevation (meters) in the tracks contained in the Gpx element.

        Returns:
            float: Minimum elevation (meters).
        """
        min_elevation = self.tracks[0].trkseg[0].trkpt[0].ele
        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    if track_point.ele < min_elevation:
                        min_elevation = track_point.ele
        return min_elevation
    
    def max_elevation(self) -> float:
        """
        Compute maximum elevation (meters) in the tracks contained in the Gpx element.

        Returns:
            float: Maximum elevation (meters).
        """
        max_elevation = self.tracks[0].trkseg[0].trkpt[0].ele
        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    if track_point.ele > max_elevation:
                        max_elevation = track_point.ele
        return max_elevation
    
    def utc_start_time(self) -> datetime:
        """
        Return the activity UTC starting time.

        Returns:
            datetime: UTC start time.
        """
        return self.tracks[0].trkseg[0].trkpt[0].time
    
    def utc_stop_time(self):
        """
        Return the activity UTC stopping time.

        Returns:
            datetime: UTC stop time.
        """
        return self.tracks[-1].trkseg[-1].trkpt[-1].time
    
    def start_time(self) -> datetime:
        """
        Return the activity starting time.

        Returns:
            datetime: Start time.
        """
        start_time = None
        try:
            start_time = self.tracks[0].trkseg[0].trkpt[0].time.replace(tzinfo=timezone.utc).astimezone(tz=None) 
        except:
            logging.error("Unable to find activity start time")
        return start_time
    
    def stop_time(self) -> datetime:
        """
        Return the activity stopping time.

        Returns:
            datetime: Stop time.
        """
        stop_time = None
        try:
            stop_time = self.tracks[-1].trkseg[-1].trkpt[-1].time.replace(tzinfo=timezone.utc).astimezone(tz=None)
        except:
            logging.error("Unable to find activity stop time")
        return stop_time
    
    def total_elapsed_time(self) -> datetime:
        """
        Compute the total elapsed time.

        Returns:
            datetime: Total elapsed time.
        """
        total_elapsed_time = None
        try:
            total_elapsed_time = self.stop_time() - self.start_time()
        except:
            logging.error("Unable to compute activity total elapsed time")
        return total_elapsed_time
    
    def stopped_time(self, tolerance: float = 2.45) -> datetime:
        """
        Compute the stopped time during activity.

        Args:
            tolerance (float, optional): Maximal distance between two points for movement. Defaults to 2.45. (According to my tests with strava_run_1 and the data on Strava).

        Returns:
            datetime: Stopped time.
        """
        stopped_time = self.start_time() - self.start_time() # Better way to do it?

        previous_point = self.tracks[0].trkseg[0].trkpt[0]

        for track in self.tracks:
            for segment in track.trkseg:
                for point in segment.trkpt:
                    if haversine_distance(previous_point, point) < tolerance:
                        stopped_time += point.time - previous_point.time
                    previous_point = point

        return stopped_time
    
    def moving_time(self) -> datetime:
        """
        Compute the moving time during the activity.

        Returns:
            datetime: Moving time.
        """
        return self.total_elapsed_time() - self.stopped_time()
    
    def avg_speed(self) -> float:
        """
        Compute the average speed (kilometers per hour) during the activity.

        Returns:
            float: Average speed (kilometers per hour).
        """
        # Compute and convert total elapsed time
        total_elapsed_time = self.total_elapsed_time()
        total_elapsed_time = total_elapsed_time.total_seconds() / 3600

        # Compute and convert distance
        distance = self.distance() / 1000

        return distance/total_elapsed_time
    
    def avg_moving_speed(self) -> float:
        """
        Compute the average moving speed (kilometers per hour) during the activity.

        Returns:
            float: Average moving speed (kilometers per hour).
        """
        # Compute and convert moving time
        moving_time = self.moving_time()
        moving_time = moving_time.total_seconds() / 3600

        # Compute and convert distance
        distance = self.distance() / 1000

        return distance / moving_time
    
    def avg_pace(self) -> float:
        """
        Compute the average pace (minute per kilometer) during the activity.

        Returns:
            float: Average pace (minute per kilometer).
        """
        return 60 / self.avg_speed()
    
    def avg_moving_pace(self) -> float:
        """
        Compute the average moving pace (minute per kilometer) during the activity.

        Returns:
            float: Average moving pace (minute per kilometer).
        """
        return 60 / self.avg_moving_speed()

    def to_dataframe(self) -> pd.DataFrame:
        """
        Convert Gpx element to Pandas Dataframe.

        Returns:
            pd.DataFrame: Dataframe containing position data from GPX.
        """
        route_info = []
        for track in self.tracks:
            for segment in track.trkseg:
                for point in segment.trkpt:
                    route_info.append({
                        "latitude": point.lat,
                        "longitude": point.lon,
                        "elevation": point.ele,
                        "x": point._x,
                        "y": point._y
                    })
        df = pd.DataFrame(route_info)
        return df
    
    def project(self, projection: str):
        """
        Project tracks.

        Args:
            projection (str): Projection.
        """
        for track in self.tracks:
            track.project(projection)
    
    def remove_gps_errors(self, error_distance=100):
        """
        Remove GPS errors.

        Args:
            error_distance (int, optional): Error threshold distance (meters) between two points. Defaults to 1000.

        Returns:
            list: List of removed points (GPS errors).
        """
        previous_point = None
        gps_errors = []

        for track in self.tracks:
            for track_segment in track.trkseg:

                new_trkpt = []

                for track_point in track_segment.trkpt:
                    # GPS error
                    if previous_point is not None and haversine_distance(previous_point,
                                                                         track_point) > error_distance:
                        logging.warning(f"Point {track_point} has been removed (GPS error)")
                        gps_errors.append(track_point)
                    # No GPS error
                    else:
                        new_trkpt.append(track_point)
                        previous_point = track_point

                track_segment.trkpt = new_trkpt

        return gps_errors
    
    def remove_close_points(self, min_dist: float = 1, max_dist: float = 10):
        """
        Remove points that are to close together.

        Args:
            min_dist (float, optional): Minimal distance between two points. Defaults to 1.
            max_dist (float, optional): Maximal distance between two points. Defaults to 10.
        """
        point_1 = None
        point_2 = None
        
        for track in self.tracks:
            for segment in track.trkseg:

                new_trkpt = []

                for point in segment.trkpt:
                    if point_1 is None:
                        point_1 = point
                        new_trkpt.append(point_1)
                    elif point_2 is None:
                        point_2 = point
                    else:
                        if ((haversine_distance(point_1, point_2) < min_dist
                             or haversine_distance(point_2, point) < min_dist)
                            and haversine_distance(point_1, point) < max_dist):
                            point_2 = point
                        else:
                            new_trkpt.append(point_2)
                            point_1 = point_2
                            point_2 = point

                segment.trkpt = new_trkpt

    def remove_points(self, remove_factor: int = 2):
        count = 0
        for track in self.tracks:
            for track_segment in track.trkseg:
                for track_point in track_segment.trkpt:
                    if count % remove_factor == 0:
                        track_segment.trkpt.remove(track_point)
                        count += 1

    def simplify(self, epsilon):
        """
        Simplify GPX tracks using Ramer-Douglas-Peucker algorithm.

        Args:
            epsilon (float): Tolerance.
        """
        for track in self.tracks:
            for segment in track.trkseg:
                segment.trkpt = ramer_douglas_peucker(segment.trkpt, epsilon)


import sys
import os
import pytest
from shutil import rmtree

import numpy as np
import math
import matplotlib.pyplot as plt

file_folder = os.path.dirname(__file__)
parent_folder = os.path.realpath(os.path.dirname(file_folder)) # ie: ezGPX

os.chdir(file_folder)
sys.path.append(parent_folder + "/ezgpx")

from ezgpx import utils, WayPoint

class TestUtils():

    def test_haversine_distance(self):
        pass

    def _test_perpendicular_distance_horizontal_line(self):
        start = WayPoint("wpt", 0, 0)
        end = WayPoint("wpt", 0, 2)
        point = WayPoint("wpt", 1, 1)
        return math.isclose(utils.perpendicular_distance(start, end, point), 1)

    def _test_perpendicular_distance_vertical_line(self):
        start = WayPoint("wpt", 0, 0)
        end = WayPoint("wpt", 2, 0)
        point = WayPoint("wpt", 1, 1)
        return math.isclose(utils.perpendicular_distance(start, end, point), 1)
    
    def _test_perpendicular_distance_random_line(self):
        start = WayPoint("wpt", 0, 0)
        end = WayPoint("wpt", 1, 1)
        point = WayPoint("wpt", 1, 0)
        return math.isclose(utils.perpendicular_distance(start, end, point), math.sqrt(2)/2)
    
    def _test_perpendicular_distance_point_on_line(self):
        start = WayPoint("wpt", 0, 0)
        end = WayPoint("wpt", 1, 1)
        point = WayPoint("wpt", 2, 2)
        return math.isclose(utils.perpendicular_distance(start, end, point), 0)
    
    def test_perpendicular_distance(self):
        assert self._test_perpendicular_distance_horizontal_line()
        assert self._test_perpendicular_distance_vertical_line()
        assert self._test_perpendicular_distance_random_line()
        assert self._test_perpendicular_distance_point_on_line()



import logging
import datetime

from .extensions import Extensions
from .link import Link
from ..utils import web_mercator_projection, lambert_conformal_conic_projection

class WayPoint():
    """
    wptType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "wpt",
            lat: float = None,
            lon: float = None,
            ele: float = None,
            time: datetime = None,
            mag_var: float = None, # 0 <= value < 360
            geo_id_height: float = None,
            name: str = None,
            cmt: str = None,
            desc: str = None,
            src: str = None,
            link: Link = None,
            sym: str = None,
            type: str = None,
            fix: str = None, # none, 2d, 3d, dgps, pps
            sat: int = None, # non negative integer
            hdop: float = None,
            vdop: float = None,
            pdop: float = None,
            age_of_gps_data: float = None,
            dgpsid: int = None, # 0 <= value <= 1023
            extensions: Extensions = None) -> None:
        """
        Initialize WayPoint instance.

        Args:
            tag (str, optional): XML tag. Defaults to "wpt".
            lat (float, optional): Latitude. Defaults to None.
            lon (float, optional): Longitude. Defaults to None.
            ele (float, optional): Elevation. Defaults to None.
            time (datetime, optional): Time. Defaults to None.
            mag_var (float, optional): _description_. Defaults to None.
            geo_id_height (float, optional): _description_. Defaults to None.
            name (str, optional): Name. Defaults to None.
            cmt (str, optional): Comment. Defaults to None.
            desc (str, optional): Description. Defaults to None.
            src (str, optional): Source. Defaults to None.
            link (Link, optional): link. Defaults to None.
            sym (str, optional): _description_. Defaults to None.
            type (str, optional): Type. Defaults to None.
            fix (str, optional): _description_. Defaults to None.
            ppssat (int, optional): _description_. Defaults to None.
            vdop (float, optional): _description_. Defaults to None.
            pdop (float, optional): _description_. Defaults to None.
            age_of_gps_data (float, optional): Age of GPS data. Defaults to None.
            dgpsid (int, optional): _description_. Defaults to None.
        """
        self.tag: str = tag
        self.lat: float = lat
        self.lon: float = lon
        self.ele: float = ele
        self.time: datetime = time
        self.mag_var: float = mag_var
        self.geo_id_height: float = geo_id_height
        self.name: str = name
        self.cmt: str = cmt
        self.desc: str = desc
        self.src: str = src
        self.link: Link = link
        self.sym: str = sym
        self.type: str = type
        self.fix: str = fix
        self.sat: int = sat
        self.hdop: float = hdop
        self.vdop: float = vdop
        self.pdop: float = pdop
        self.age_of_gps_data: float = age_of_gps_data
        self.dgpsid: int = dgpsid
        self.extensions: Extensions = extensions

        self._x: int = None
        self._y: int = None

    def project(self, projection: str):
        """
        Project point.

        Args:
            projection (str): Projection.
        """
        if projection in ["web_mercator_projection", "web_mercator", "wm"]:
            self._x, self._y = web_mercator_projection(self.lat, self.lon)
        elif projection in ["lambert_conformal_conic_projection", "lambert_conformal_conic", "lcc"]:
            self._x, self._y = lambert_conformal_conic_projection(self.lat, self.lon, 90.0, 135.0, 45.0, 45.0)
        else:
            logging.error(f"Invalid projection: {projection}")



import os
from typing import Optional, Union
import logging
import webbrowser
from datetime import datetime
import pandas as pd
from math import degrees

from matplotlib.figure import Figure
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap

import gmplot

import folium
from folium.plugins import MiniMap
from folium.features import DivIcon

from ..gpx_elements import Gpx, WayPoint
from ..gpx_parser import Parser
from ..gpx_writer import Writer
from ..utils import EARTH_RADIUS


class GPX():
    """
    High level GPX object.
    """

    def __init__(self, file_path: str):
        self.file_path: str = file_path
        self.parser: Parser = Parser(file_path)
        self.gpx: Gpx = self.parser.gpx
        self.writer: Writer = Writer(
            self.gpx, precisions=self.parser.precisions, time_format=self.parser.time_format)

    def name(self) -> str:
        """
        Return activity name.

        Returns:
            str: Activity name.
        """
        return self.gpx.name()

    def nb_points(self) -> int:
        """
        Return the number of points in the GPX.

        Returns:
            int: Number of points in the GPX.
        """
        return self.gpx.nb_points()

    def first_point(self) -> WayPoint:
        """
        Return GPX first point.

        Returns:
            WayPoint: First point.
        """
        return self.gpx.tracks[0].trkseg[0].trkpt[0]

    def last_point(self) -> WayPoint:
        """
        Return GPX last point.

        Returns:
            WayPoint: Last point.
        """
        return self.gpx.tracks[-1].trkseg[-1].trkpt[-1]

    def bounds(self) -> tuple[float, float, float, float]:
        """
        Find minimum and maximum latitude and longitude.

        Returns:
            tuple[float, float, float, float]: Min latitude, min longitude, max latitude, max longitude
        """
        return self.gpx.bounds()

    def center(self) -> tuple[float, float]:
        """
        Return the coordinates of the center point.

        Returns:
            tuple[float, float]: Latitude and longitude of the center point.
        """
        return self.gpx.center()

    def distance(self) -> float:
        """
        Returns the distance (meters) of the tracks contained in the GPX.

        Returns:
            float: Distance (meters).
        """
        return self.gpx.distance()

    def ascent(self) -> float:
        """
        Returns the ascent (meters) of the tracks contained in the GPX.

        Returns:
            float: Ascent (meters).
        """
        return self.gpx.ascent()

    def descent(self) -> float:
        """
        Returns the descent (meters) of the tracks contained in the GPX.

        Returns:
            float: Descent (meters).
        """
        return self.gpx.descent()

    def min_elevation(self) -> float:
        """
        Returns the minimum elevation (meters) in the tracks contained in the GPX.

        Returns:
            float: Minimum elevation (meters).
        """
        return self.gpx.min_elevation()

    def max_elevation(self) -> float:
        """
        Returns the maximum elevation (meters) in the tracks contained in the GPX.

        Returns:
            float: Maximum elevation (meters).
        """
        return self.gpx.max_elevation()

    def start_time(self) -> datetime:
        """
        Return the activity start time.

        Returns:
            datetime: Start time.
        """
        return self.gpx.start_time()

    def stop_time(self) -> datetime:
        """
        Return the activity stop time.

        Returns:
            datetime: Stop time.
        """
        return self.gpx.stop_time()

    def total_elapsed_time(self) -> datetime:
        """
        Return the total elapsed time during the activity.

        Returns:
            datetime: Total elapsed time.
        """
        return self.gpx.total_elapsed_time()

    def stopped_time(self) -> datetime:
        """
        Return the stopped time during the activity.

        Returns:
            datetime: Stopped time.
        """
        return self.gpx.stopped_time()

    def moving_time(self) -> datetime:
        """
        Return the moving time during the activity.

        Returns:
            datetime: Moving time.
        """
        return self.gpx.moving_time()

    def avg_speed(self) -> float:
        """
        Return average speed (kilometers per hour) during the activity.

        Returns:
            float: Average speed (kilometers per hour).
        """
        return self.gpx.avg_speed()

    def avg_moving_speed(self) -> float:
        """
        Return average moving speed (kilometers per hour) during the activity.

        Returns:
            float: Average moving speed (kilometers per hour).
        """
        return self.gpx.avg_moving_speed()

    def max_speed(self) -> float:
        # TODO
        pass

    def avg_pace(self) -> float:
        """
        Return average pace (minutes per kilometer) during the activity.

        Returns:
            float: Average pace (minutes per kilometer).
        """
        return self.gpx.avg_pace()

    def avg_moving_pace(self) -> float:
        """
        Return average moving pace (minutes per kilometer) during the activity.

        Returns:
            float: Average moving pace (minutes per kilometer).
        """
        return self.gpx.avg_moving_pace()

    def to_string(self) -> str:
        """
        Convert the GPX object to a string.

        Returns:
            str: String representingth GPX object.
        """
        return self.writer.gpx_to_string(self.gpx)

    def to_gpx(self, path: str):
        """
        Write the GPX object to a .gpx file.

        Args:
            path (str): Path to the .gpx file.
        """
        self.writer.write(path)

    def to_dataframe(self) -> pd.DataFrame:
        """
        Convert GPX object to Pandas Dataframe.

        Returns:
            pd.DataFrame: Dataframe containing position data from GPX.
        """
        return self.gpx.to_dataframe()

    def remove_metadata(self):
        """
        Remove metadata (ie: metadata will not be written when saving the GPX object as a .gpx file).
        """
        self.writer.metadata = False

    def remove_elevation(self):
        """
        Remove elevation data (ie: elevation data will not be written when saving the GPX object as a .gpx file).
        """
        self.writer.ele = False

    def remove_time(self):
        """
        Remove time data (ie: time data will not be written when saving the GPX object as a .gpx file).
        """
        self.writer.time = False

    def remove_gps_errors(self):
        """
        Remove GPS errors.
        """
        self.gpx.remove_gps_errors()

    def remove_close_points(self, min_dist: float = 1, max_dist: float = 10):
        """
        Remove points that are to close together.

        Args:
            min_dist (float, optional): Minimal distance between two points. Defaults to 1.
            max_dist (float, optional): Maximal distance between two points. Defaults to 10.
        """
        self.gpx.remove_close_points(min_dist, max_dist)

    def simplify(self, tolerance: float = 2):
        """
        Simplify the tracks using Ramer-Douglas-Peucker algorithm.

        Args:
            tolerance (float, optional): Tolerance (meters). Corresponds to the
            minimum distance between the point and the track before the point
            is removed. Defaults to 2.
        """
        epsilon = degrees(tolerance/EARTH_RADIUS)
        self.gpx.simplify(epsilon)

    def _matplotlib_plot_text(
            self,
            fig: Figure,
            duration: Optional[tuple[float, float]] = (0, 0),
            distance: Optional[tuple[float, float]] = (0.5, 0),
            ascent: Optional[tuple[float, float]] = (1, 0),
            pace: Optional[tuple[float, float]] = (1, 0),
            speed: Optional[tuple[float, float]] = (1, 0),
            text_parameters: Optional[dict] = None):
        """
        Plot text on Matplotlib plot.

        Args:
            fig (matplotlib.figure.Figure): Matplotlib figure.
            duration (Optional[tuple[float, float]], optional): Duration text plot coordinates. Defaults to (0, 0).
            distance (Optional[tuple[float, float]], optional): Distance text plot coordinates. Defaults to (0.5, 0).
            ascent (Optional[tuple[float, float]], optional): Ascent text plot coordinates. Defaults to (1, 0).
            pace (Optional[tuple[float, float]], optional): Pace text plot coordinates. Defaults to (1, 0).
            speed (Optional[tuple[float, float]], optional): Speed text plot coordinates. Defaults to (1, 0).
            text_parameters (Optional[dict], optional): Text parameters. Defaults to None.
        """
        # Handle text parameters
        if text_parameters is None:
            text_parameters = dict(ha='center', va='center', fontsize=10,
                                   transform=fig.axes[0].transAxes, bbox=dict(facecolor='gray', alpha=0.5))
        else:
            text_parameters["transform"] = fig.axes[0].transAxes

        if duration is not None:
            plt.text(duration[0], duration[1],
                     f"Duration:\n{self.total_elapsed_time()}", **text_parameters)

        if distance is not None:
            plt.text(distance[0], distance[1],
                     f"Distance:\n{self.distance()/1000:.2f} km", **text_parameters)

        if ascent is not None:
            plt.text(ascent[0], ascent[1],
                     f"Ascent:\n{self.ascent():.2f} m", **text_parameters)

        elif pace is not None:
            plt.text(pace[0], pace[1],
                     f"Pace:\n{self.avg_pace():.2f} min/km", **text_parameters)

        elif speed is not None:
            plt.text(speed[0], speed[1],
                     f"Speed:\n{self.avg_speed():.2f} km/h", **text_parameters)

    def matplotlib_plot(
            self,
            projection: Optional[str] = None,
            base_color: str = "#101010",
            elevation_color: bool = False,
            start_stop_colors: Optional[tuple[str, str]] = None,
            way_points_color: Optional[str] = None,
            title: Optional[str] = None,
            duration: Optional[tuple[float, float]] = None,
            distance: Optional[tuple[float, float]] = None,
            ascent: Optional[tuple[float, float]] = None,
            pace: Optional[tuple[float, float]] = None,
            speed: Optional[tuple[float, float]] = None,
            file_path: Optional[str] = None):
        """
        Plot GPX using Matplotlib.

        Args:
            projection (Optional[str], optional): Projection to use. Defaults to None.
            base_color (str, optional): Track color. Defaults to "#101010".
            elevation_color (bool, optional): Plot track with color according to elevation. Defaults to False.
            start_stop_colors (tuple[str, str], optional): Start and stop points colors. Defaults to False.
            way_points_color (str, optional): Way point color. Defaults to False.
            title (Optional[str], optional): Title. Defaults to None.
            duration (Optional[tuple[float, float]], optional): Display duration. Defaults to None.
            distance (Optional[tuple[float, float]], optional): Display distance. Defaults to None.
            ascent (Optional[tuple[float, float]], optional): Display ascent. Defaults to None.
            pace (Optional[tuple[float, float]], optional): Display pace. Defaults to None.
            speed (Optional[tuple[float, float]], optional): Display pace. Defaults to None.
            file_path (Optional[str], optional): Path to save plot. Defaults to None.
        """
        # Handle projection (select dataframe columns to use and project if needed)
        if projection is None:
            column_x = "longitude"
            column_y = "latitude"
        else:
            column_x = "x"
            column_y = "y"
            self.gpx.project(projection)  # Project all track points

        # Create dataframe containing data from the GPX file
        gpx_df = self.to_dataframe()

        # Create figure
        fig = plt.figure(figsize=(14, 8))

        # Scatter all track points
        if elevation_color:
            plt.scatter(gpx_df[column_x], gpx_df[column_y],
                        c=gpx_df["elevation"])
        else:
            plt.scatter(gpx_df[column_x], gpx_df[column_y], color=base_color)

        # Scatter start and stop points with different color
        if start_stop_colors is not None:
            plt.scatter(gpx_df[column_x][0],
                        gpx_df[column_y][0], color=start_stop_colors[0])
            plt.scatter(gpx_df[column_x][len(gpx_df[column_x])-1],
                        gpx_df[column_y][len(gpx_df[column_x])-1], color=start_stop_colors[1])

        # Scatter way points with different color
        if way_points_color:
            for way_point in self.gpx.wpt:
                if projection:
                    way_point.project(projection)
                    plt.scatter(way_point._x, way_point._y, color=way_points_color)
                else:
                    plt.scatter(way_point.lon, way_point.lat, color=way_points_color)

        # Add title
        if title is not None:
            plt.title(title, size=20)

        # Add text elements
        self._matplotlib_plot_text(fig, duration, distance, ascent, pace, speed)

        # Add ticks
        plt.xticks([min(gpx_df[column_x]), max(gpx_df[column_x])])
        plt.yticks([min(gpx_df[column_y]), max(gpx_df[column_y])])

        # Add axis limits (useless??)
        if projection is not None:
            ax = plt.gca()
            ax.set_xlim(left=min(gpx_df[column_x]),
                        right=max(gpx_df[column_x]))
            ax.set_ylim(bottom=min(gpx_df[column_y]),
                        top=max(gpx_df[column_y]))

        # Save or display plot
        if file_path is not None:
            directory_path = os.path.dirname(os.path.realpath(file_path))
            if not os.path.exists(directory_path):
                logging.error("Provided path does not exist")
                return
            plt.savefig(file_path)
        else:
            plt.show()

    def matplotlib_basemap_plot(
            self,
            projection: str = "cyl",
            service: str = "World_Shaded_Relief",
            base_color: str = "#101010",
            start_stop_colors: Optional[tuple[str, str]] = None,
            way_points_color: Optional[str] = None,
            title: Optional[str] = None,
            duration: Optional[tuple[float, float]] = None,
            distance: Optional[tuple[float, float]] = None,
            ascent: Optional[tuple[float, float]] = None,
            pace: Optional[tuple[float, float]] = None,
            speed: Optional[tuple[float, float]] = None,
            file_path: str = None):
        """
        Plot GPX using Matplotlib Basemap Toolkit.

        Args:
            projection (str, optional): Projection. Currently supported projections: cyl. Defaults to "cyl".
            service (str, optional): Service used to fetch map background. Currently supported services: "World_Shaded_Relief". Defaults to "World_Shaded_Relief".
            base_color (str, optional): Track color. Defaults to "#101010".
            start_stop_colors (tuple[str, str], optional): Start and stop points colors. Defaults to False.
            way_points_color (str, optional): Way point color. Defaults to False.
            title (Optional[str], optional): Title. Defaults to None.
            duration (Optional[tuple[float, float]], optional): Display duration. Defaults to None.
            distance (Optional[tuple[float, float]], optional): Display distance. Defaults to None.
            ascent (Optional[tuple[float, float]], optional): Display ascent. Defaults to None.
            pace (Optional[tuple[float, float]], optional): Display pace. Defaults to None.
            speed (Optional[tuple[float, float]], optional): Display pace. Defaults to None.
            file_path (Optional[str], optional): Path to save plot. Defaults to None.
        """
        # Create figure
        fig = plt.figure(figsize=(14, 8))

        # Create square map
        center_lat, center_lon = self.center()
        min_lat, min_lon, max_lat, max_lon = self.bounds()
        offset = 0.001
        min_lat, min_lon = max(0, min_lat - offset), max(0, min_lon - offset)
        max_lat, max_lon = min(max_lat + offset, 90), min(max_lon + offset, 90)

        delta_lat = max_lat - min_lat
        delta_lon = max_lon - min_lon

        if delta_lat > delta_lon:
            min_lon = max(0, center_lon - delta_lat/2)
            max_lon = min(90, center_lon + delta_lat/2)
        else:
            min_lat = max(0, center_lat - delta_lon/2)
            max_lat = min(90, center_lat + delta_lon/2)

        map = Basemap(projection=projection,
                      llcrnrlon=min_lon,
                      llcrnrlat=min_lat,
                      urcrnrlon=max_lon,
                      urcrnrlat=max_lat)

        # map.drawmapboundary(fill_color='aqua')
        # map.fillcontinents(color='coral',lake_color='aqua')
        # map.drawcoastlines()

        # map.bluemarble()
        # map.shadedrelief()
        # map.etopo()
        map.arcgisimage(service=service,
                        xpixels=2000, dpi=200, verbose=True)

        # Create dataframe containing data from the GPX file
        gpx_df = self.to_dataframe()

        # Project track points
        x, y = map(gpx_df["longitude"], gpx_df["latitude"])
        
        # Plot track points on the map
        x = x.tolist()
        y = y.tolist()
        map.plot(x, y, marker=None, color=base_color)

        # Scatter start and stop points with different color
        if start_stop_colors:
            map.plot(x[0], y[0], marker="^", color=start_stop_colors[0])
            map.plot(x[-1], y[-1], marker="h", color=start_stop_colors[1])

        # Project and scatter way points with different color
        if way_points_color:
            for way_point in self.gpx.wpt:
                x, y = map(way_point.lon, way_point.lat)
                map.plot(x, y, marker="D", color="waypoint_color")

        # Add title
        if title is not None:
            plt.title(title, size=20)

        # Add text elements
        self._matplotlib_plot_text(fig, duration, distance, ascent, pace, speed)

        # Save or display plot
        if file_path is not None:
            # Check if provided path exists
            directory_path = os.path.dirname(os.path.realpath(file_path))
            if not os.path.exists(directory_path):
                logging.error("Provided path does not exist")
                return
            plt.savefig(file_path)
        else:
            plt.show()

    def gmap_plot(
            self,
            base_color: str = "#110000",
            start_stop_colors: Optional[tuple[str, str]] = None,
            way_points_color: Optional[str] = None,
            zoom: float = 10.0,
            title: Optional[str] = None,
            file_path: str = None,
            open: bool = True,
            scatter: bool = False,
            plot: bool = True):
        """
        Plot GPX using gmap.

        Args:
            base_color (str, optional): Track_color. Defaults to "#110000".
            start_stop_colors (tuple[str, str], optional): Start and stop points colors. Defaults to False.
            way_points_color (str, optional): Way point color. Defaults to False.
            zoom (float, optional): Zoom. Defaults to 10.0.
            title (str, optional): Title. Defaults to None.
            file_path (str, optional): Path to save plot. Defaults to None.
            open (bool, optional): Open the plot in the default web browser. Defaults to True.
            scatter (bool, optional): Scatter track points. Defaults to False.
            plot (bool, optional): Plot track points. Defaults to True.
        """
        # Create plotter
        center_lat, center_lon = self.center()
        map = gmplot.GoogleMapPlotter(center_lat, center_lon, zoom)

        # Create dataframe containing data from the GPX file
        gpx_df = self.to_dataframe()

        # Scatter track points
        if scatter:
            map.scatter(gpx_df["latitude"], gpx_df["longitude"],
                        base_color, size=5, marker=False)
        if plot:
            map.plot(gpx_df["latitude"], gpx_df["longitude"],
                     base_color, edge_width=2.5)
        
        # Scatter start and stop points with different color
        if start_stop_colors:
            map.scatter([self.gpx.tracks[0].trkseg[0].trkpt[0].lat],
                        [self.gpx.tracks[0].trkseg[0].trkpt[0].lon],
                        start_stop_colors[0], size=5, marker=True)
            map.scatter([self.gpx.tracks[-1].trkseg[-1].trkpt[-1].lat],
                        [self.gpx.tracks[-1].trkseg[-1].trkpt[-1].lon],
                        start_stop_colors[1], size=5, marker=True)
            
        # Scatter way points with different color
        if way_points_color:
            for way_point in self.gpx.wpt:
                map.scatter([way_point.lat], [way_point.lon],
                            way_points_color, size=5, marker=True)
        
        # Add title
        if title is not None:
            map.text(center_lat, center_lon, self.name(), color="#FFFFFF")

        # Save map
        map.draw(file_path)

        # Open map in web browser
        if open:
            webbrowser.open(file_path)

    def folium_plot(
            self,
            tiles: str = "OpenStreetMap",  # "OpenStreetMap", "Stamen Terrain", "Stamen Toner"
            base_color: str = "#110000",
            start_stop_colors: Optional[tuple[str, str]] = None,
            way_points_color: Optional[str] = None,
            minimap: bool = False,
            coord_popup: bool = False,
            title: Optional[str] = None,
            zoom: float = 12.0,
            file_path: str = None,
            open: bool = True):
        """
        Plot GPX using folium.

        Args:
            tiles (str, optional): Map tiles. Supported tiles: "OpenStreetMap", "Stamen Terrain", "Stamen Toner". Defaults to "OpenStreetMap".
            start_stop_colors (tuple[str, str], optional): Start and stop points colors. Defaults to False.
            way_points_color (str, optional): Way point color. Defaults to False.
            minimap (bool, optional): Add minimap. Defaults to False.
            coord_popup (bool, optional): Add coordinates pop-up when clicking on the map. Defaults to False.
            title (Optional[str], optional): _description_. Defaults to None.
            zoom (float, optional): Zoom. Defaults to 12.0.
            file_path (str, optional): Path to save plot. Defaults to None.
            open (bool, optional): Open the plot in the default web browser. Defaults to True.
        """
        # Create map
        center_lat, center_lon = self.center()
        m = folium.Map(location=[center_lat, center_lon],
                       zoom_start=zoom,
                       tiles=tiles)
        
        # Plot track points
        gpx_df = self.to_dataframe()
        gpx_df["coordinates"] = list(
            zip(gpx_df.latitude, gpx_df.longitude))
        folium.PolyLine(gpx_df["coordinates"],
                        tooltip=self.name(), color=base_color).add_to(m)

        # Scatter start and stop points with different color
        if start_stop_colors:
            folium.Marker([self.gpx.tracks[0].trkseg[0].trkpt[0].lat, self.gpx.tracks[0].trkseg[0].trkpt[0].lon],
                          popup="<b>Start</b>", tooltip="Start", icon=folium.Icon(color=start_stop_colors[0])).add_to(m)
            folium.Marker([self.gpx.tracks[-1].trkseg[-1].trkpt[-1].lat, self.gpx.tracks[-1].trkseg[-1].trkpt[-1].lon],
                          popup="<b>Stop</b>", tooltip="Stop", icon=folium.Icon(color=start_stop_colors[1])).add_to(m)
        
        # Scatter way points with different color
        if way_points_color:
            for way_point in self.gpx.wpt:
                folium.Marker([way_point.lat, way_point.lon], popup="<i>Way point</i>",
                              tooltip="Way point", icon=folium.Icon(icon="info-sign", color=way_points_color)).add_to(m)

        # Add minimap
        if minimap:
            minimap = MiniMap(toggle_display=True)
            minimap.add_to(m)

        # Add latitude-longitude pop-up
        if coord_popup:
            m.add_child(folium.LatLngPopup())

        # Title
        if title is not None:
            folium.map.Marker(
                [center_lat, center_lon],
                icon=DivIcon(
                    icon_size=(250, 36),
                    icon_anchor=(0, 0),
                    html=f'<div style="font-size: 20pt">{title}</div>',
                )
            ).add_to(m)

        # Save map
        m.save(file_path)

        # Open map in web browser
        if open:
            webbrowser.open(file_path)




class Bounds():
    """
    boundsType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "bounds",
            minlat: float = None,
            minlon: float = None,
            maxlat: float = None,
            maxlon: float = None) -> None:
        """
        Initialize Bounds instance.

        Args:
            tag (str, optional): XML tag. Defaults to "bounds".
            minlat (float, optional): Minimal latitude. Defaults to None.
            minlon (float, optional): Minimal longitude. Defaults to None.
            maxlat (float, optional): Maximal latitude. Defaults to None.
            maxlon (float, optional): Maximal longitude. Defaults to None.
        """
        self.tag: str = tag
        self.minlat: float = minlat
        self.minlon: float = minlon
        self.maxlat: float = maxlat
        self.maxlon: float = maxlon


import logging

from .extensions import Extensions
from .link import Link
from .way_point import WayPoint

class Route():
    """
    rteType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "rte",
            name: str = None,
            cmt: str = None,
            desc: str = None,
            src: str = None,
            link: Link = None,
            number: int = None, # non negative integer
            type: str = None,
            extensions: Extensions = None,
            rtept: list[WayPoint] = []) -> None:
        """
        Initialize Route instance.

        Args:
            tag (str, optional): XML tag. Defaults to "rte".
            name (str, optional): Name. Defaults to None.
            cmt (str, optional): Comment. Defaults to None.
            desc (str, optional): Description. Defaults to None.
            src (str, optional): Source. Defaults to None.
            link (Link, optional): Link. Defaults to None.
            number (int, optional): Number. Defaults to None.
            extensions (Extensions, optional): Extensions. Defaults to None.
            rtept (list[WayPoint], optional): Route points. Defaults to [].
        """
        self.tag: str = tag
        self.name: str = name
        self.cmt: str = cmt
        self.desc: str = desc
        self.src: str = src
        self.link: Link = link
        self.number: int = number
        self.type: str = type
        self.extensions: Extensions = extensions
        self.rtept: list[WayPoint] = rtept


import sys
import os
import pytest
from shutil import rmtree

import numpy as np
import matplotlib.pyplot as plt

file_folder = os.path.dirname(__file__)
parent_folder = os.path.realpath(os.path.dirname(file_folder)) # ie: ezGPX

os.chdir(file_folder)
sys.path.append(parent_folder + "/ezgpx")

from ezgpx import GPX

class TestGPX():

    def _test_matplotlib_plot_1(self):
        # Plot
        self.test_gpx.matplotlib_plot(start_stop_colors=None, elevation_color=False, title="Track", file_path="tmp/matplotlib_strava_run_1.png")

        # Load images
        test_img = plt.imread("tmp/matplotlib_strava_run_1.png")
        ref_img = plt.imread("../tests/test_files/reference_files/matplotlib_strava_run_1.png")

        # Compare images
        return np.array_equal(test_img, ref_img)
    
    def _test_matplotlib_plot_2(self):
        # Plot
        self.test_gpx.matplotlib_plot(start_stop_colors=("#00FF00", "#FF0000"), elevation_color=False, title="Track", file_path="tmp/matplotlib_strava_run_1_start_stop.png")

        # Load images
        test_img = plt.imread("tmp/matplotlib_strava_run_1_start_stop.png")
        ref_img = plt.imread("../tests/test_files/reference_files/matplotlib_strava_run_1_start_stop.png")

        # Compare images
        return np.array_equal(test_img, ref_img)
    
    def _test_matplotlib_plot_3(self):
        # Plot
        self.test_gpx.matplotlib_plot(start_stop_colors=None, elevation_color=True, title="Track", file_path="tmp/matplotlib_strava_run_1_elevation.png")

        # Load images
        test_img = plt.imread("tmp/matplotlib_strava_run_1_elevation.png")
        ref_img = plt.imread("../tests/test_files/reference_files/matplotlib_strava_run_1_elevation.png")

        # Compare images
        return np.array_equal(test_img, ref_img)
    
    def _test_matplotlib_plot_4(self):
        # Plot
        self.test_gpx.matplotlib_plot(start_stop_colors=("#00FF00", "#FF0000"), elevation_color=True, title="Track", file_path="tmp/matplotlib_strava_run_1_start_stop_elevation.png")

        # Load images
        test_img = plt.imread("tmp/matplotlib_strava_run_1_start_stop_elevation.png")
        ref_img = plt.imread("../tests/test_files/reference_files/matplotlib_strava_run_1_start_stop_elevation.png")

        # Compare images
        return np.array_equal(test_img, ref_img)

    def test_matplotlib_plot(self, remove_tmp: bool = True):
        """
        Test matplotlib_plot method.

        Args:
            remove_tmp (bool, optional): Remove temporary folder. Defaults to True.
        """
        self.test_gpx = GPX("../tests/test_files/files/strava_run_1.gpx")

        # Create temporary folder
        rmtree("tmp", True)
        os.makedirs(os.path.dirname(__file__) + "/tmp")

        # Tests
        assert(self._test_matplotlib_plot_1())
        assert(self._test_matplotlib_plot_2())
        assert(self._test_matplotlib_plot_3())
        assert(self._test_matplotlib_plot_4())

        # Remove temporary folder
        if remove_tmp:
            rmtree("tmp")

    @pytest.mark.skip(reason="test") # https://docs.pytest.org/en/7.3.x/how-to/skipping.html
    def test_test(self):
        print("Testing...")


from .extensions import Extensions
from .way_point import WayPoint

class TrackSegment():
    """
    trksegType in GPX file.
    """
    
    def __init__(
            self,
            tag: str = "trkseg",
            trkpt: list[WayPoint] = [],
            extensions: Extensions = None) -> None:
        """
        Initialize TrackSegment instance.

        Args:
        tag (str, optional): XML tag. Defaults to "trkseg".
            trkpt (list[WayPoint], optional): List of track points. Defaults to [].
            extensions (Extensions, optional): Extensions. Defaults to None.
        """
        self.tag: str = tag
        self.trkpt: list[WayPoint] = trkpt
        self.extensions: Extensions = extensions

    def project(self, projection: str):
        """
        Project points.

        Args:
            projection (str): Projection.
        """
        for point in self.trkpt:
            point.project(projection)


import os
from setuptools import setup, find_packages
from codecs import open

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the README file
with open(os.path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ezgpx',
    version='0.1.3',
    description='Easy to use Python GPX library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['gpx', 'gpx-files', 'gpx-parser', 'gpx-reader', 'gpx-writer', 'gpx-data'],
    url='https://pypi.org/project/ezgpx/',
    download_url='https://github.com/FABallemand/ezGPX',
    project_urls={
            "Bug Tracker": "https://github.com/FABallemand/ezGPX/issues",
            "Documentation": "https://ezgpx.readthedocs.io/en/latest/",
            "Source Code": "https://github.com/FABallemand/ezGPX",
        },
    author='Fabien ALLEMAND',
    author_email='allemand.fabien@orange.fr',
    license='GNU GPLv3',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Operating System :: OS Independent'
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=['gmap', 'folium'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests'
)


from .email import Email
from .link import Link

class Person():
    """
    personType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "person",
            name: str = None,
            email: Email = None,
            link: Link = None) -> None:
        """
        Initialize Person instance.

        Args:
            tag (str, optional): XML tag. Defaults to "person".
            name (str, optional): Name. Defaults to None.
            email (Email, optional): Email. Defaults to None.
            link (Link, optional): Link. Defaults to None.
        """
        self.tag: str = tag
        self.name: str = name
        self.email: Email = email
        self.link: Link = link



class Extensions():
    """
    extensionsType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "extensions",
            display_color: str = "Cyan",
            distance: float = None,
            total_elapsed_time: float = None,
            moving_time: float = None,
            stopped_time: float = None,
            moving_speed: float = None,
            max_speed: float = None,
            max_elevation: float = None,
            min_elevation: float = None,
            ascent: float = None,
            descent: float = None,
            avg_ascent_rate: float = None,
            max_ascent_rate: float = None,
            avg_descent_rate: float = None,
            max_descent_rate: float = None) -> None:
        """
        Initialize Extensions instance.

        Args:
            tag (str, optional): XML tag. Defaults to "extensions".
            display_color (str, optional): Display color. Defaults to "Cyan".
            distance (float, optional): Distance. Defaults to None.
            total_elapsed_time (float, optional): Total elapsed time. Defaults to None.
            moving_time (float, optional): Moving time. Defaults to None.
            stopped_time (float, optional): Stopped time. Defaults to None.
            moving_speed (float, optional): Moving speed. Defaults to None.
            max_speed (float, optional): Maximum speed. Defaults to None.
            max_elevation (float, optional): Maximum elevation. Defaults to None.
            min_elevation (float, optional): Minimum elevation. Defaults to None.
            ascent (float, optional): Ascent. Defaults to None.
            descent (float, optional): Descent. Defaults to None.
            avg_ascent_rate (float, optional): Average ascent rate. Defaults to None.
            max_ascent_rate (float, optional): Maximum ascent rate. Defaults to None.
            avg_descent_rate (float, optional): Average descent rate. Defaults to None.
            max_descent_rate (float, optional): Maximum descent rate. Defaults to None.
        """
        self.tag: str = tag
        self.display_color: str = display_color
        self.distance: float = distance
        self.total_elapsed_time: float = total_elapsed_time # int?
        self.moving_time: float = moving_time # int?
        self.stopped_time: float = stopped_time
        self.moving_speed: float = moving_speed
        self.max_speed: float = max_speed
        self.max_elevation: float = max_elevation
        self.min_elevation: float = min_elevation
        self.ascent: float = ascent
        self.descent: float = descent
        self.avg_ascent_rate: float = avg_ascent_rate
        self.max_ascent_rate: float = max_ascent_rate
        self.avg_descent_rate: float = avg_descent_rate
        self.max_descent_rate: float = max_descent_rate



from .point import Point

class PointSegment():
    """
    ptsegType element in GPX file.
    """

    def __init__(
            self,
            tag: str = "ptseg",
            points: list[Point] = []) -> None:
        """
        Initialize PointSegment instance.

        Args:
            tag (str, optional): XML tag. Defaults to "ptseg".
            points (list[Point], optional): List of points. Defaults to [].
        """
        self.tag: str = tag
        self.points: list[Point] = points